""" ZWebmail base classes """

import string
import poplib
import smtplib
import mimetools
import multifile
import StringIO
import re


# --- 1. Mime part management ---

class MimeException:
    def __init__(self, message):
        self.message = message
    def __repr__(self):
        return "MimeException: %s" % self.message
    def __str__(self):
        return repr(self)


class Mimepart:
    def __init__(self, headers, message, sid = '0'):
        self.headers = headers
        self.message = message 
        self.sid = sid 
    def __getitem__(self, name):
        if name in ['message','headers','sid']:
            return self.__dict__[name]
        if name == 'type':
            return self.headers.gettype()
        return self.headers[name]
    def isMultipart(self):
        return self.headers.gettype()[:10].lower() == 'multipart/'
    def getMimeDescriptor(self):
        if self.isMultiPart():
            return "mime_multipart"
        name = self.headers.getparam('name')
        if name is not None:
            if name.lower()[-4:] in ['.jpg','.gif','.png']:
                return 'mime_image'
        mime = self.headers.gettype()
        mime_dict = {
                'text/plain': 'mime_text',
                'text/html': 'mime_html',
                'message/rfc822': 'mime_mail',
                'image/': 'mime_image',
               }
        if mime_dict.has_key(mime):
            return mime_dict[mime]
        for prefix in mime_dict.keys():
            if prefix == mime[:len(prefix)]:
                return mime_dict[prefix]
        return 'mime_unknown'
        

class Contentpart(Mimepart):
    def __init__(self, headers, message, sid):
        if headers.gettype()[:10].lower() == 'multipart/':
            raise MimeException("Can't instantiate Contentpart with mime type = [%s]" % headers.gettype()[:10])
        Mimepart.__init__(self, headers, message, sid)
    def isMultiPart(self):
        return 0
    def __str__(self):
        """ For debug purposes in this release """
        return "[%s] %s..." % (self['type'], str(self['message'][:64]))


class Multipart(Mimepart):
    def __init__(self, headers, message, sid):
        if headers.gettype()[:10].lower() != 'multipart/':
            raise MimeException("Can't instantiate Multipart with mime type = [%s]" % headers.gettype()[:10])
        Mimepart.__init__(self, headers, message, sid)
    def isMultiPart(self):
        return 1
    def getBoundary(self):
        return self.headers.getparam('boundary')
    def getParts(self):
        if not hasattr(self,'parts'):
            parts = ()
            file = multifile.MultiFile(StringIO.StringIO(self.message))
            file.push(self.getBoundary())
            try:
                i = 0
                while file.next():
                    sid = '%s_%d' % (self.sid, i)
                    i += 1
                    headers = mimetools.Message(file)
                    message = StringIO.StringIO()
                    mimetools.decode(file, message, headers.getencoding())
                    message = message.getvalue()
                    if headers.gettype()[:10].lower() == 'multipart/':
                        parts += ( Multipart(headers, message, sid) ,)
                    else:
                        parts += ( Contentpart(headers, message, sid) ,)
            finally:
                file.pop()
            self.parts = parts
        return self.parts
    #def __str__(self):
        #result = "[%s] (\n" % self['type']
        #for part in self.getParts():
            #result += " %s,\n" % str(part)
        #result += ")"
        #return result

    def __str__(self):
        """ For debug purposes in this release """
        return "[%s] (...)" % self['type']


# --- 2. Mail class ---

class Mail:
    def __init__(self, mail, id):
        self.id = id
        reader = StringIO.StringIO(mail)
        headers = mimetools.Message(reader)
        data = StringIO.StringIO()
        mimetools.decode(reader, data, headers.getencoding())
        self.message = data.getvalue()
        if headers.gettype()[:10].lower() == 'multipart/':
            self.part = Multipart(headers, self.message,'part')
        else:
            self.part = Contentpart(headers, self.message,'part')

    def __getitem__(self, name):
        if name == 'author':
            fullname, address = self.part.headers.getaddr('from')
            if fullname != '':
                return fullname
            return address
        if name == 'name':
            return self.part.headers.getaddr('from')[0]
        if name == 'email':
            return self.part.headers.getaddr('from')[1]
        if name == 'when':
            m = re.search(
                    '(?:.*,)? ?([0-9]{1,2}) ?([A-Za-z]+)',
                    self.part.headers['date']
                   ).groups()
            return m[0]+' '+m[1]
        return self.part[name]

    def quoteReadableContent(self):
        result = ''
        for line in self.getReadableContent().splitlines():
            result += '> '
            result += line
            result += '\n'
        return result
                
    def getReadableContent(self):
        if not hasattr(self,'readable_content'):
            self.readable_content =  self.getPremiumContent()
        return self.readable_content

    def getHtmlContent(self):
        if not hasattr(self,'html_content'):
            self.html_content =  self.getPremiumContent('text/html')
        return self.html_content

    def getPremiumContent(self,mimetype='text/plain'):
        if self.part['type'] == mimetype:
            return self.message
        elif self.isMultipart():
            for part in self.part.getParts()[:2]:
                if part['type'] == mimetype:
                    return part.message
                elif part.isMultipart():
                    for p2 in part.getParts()[:2]:
                        if p2['type'] == mimetype:
                            return p2.message
        return ''
                
    def isMultipart(self):
        return self.part.isMultiPart()
    def __str__(self):
        """ For debug purposes in this release """
        return ("Mail(%s, %s) : " % (self['author'], self['when'])) + str(self.part)


# --- 3. Pop and Smtp management class ---

class MailAccount:
    def __init__(self, server="127.0.0.1", login="anonymous", 
            password="anonymous@127.0.0.1"):
        self.login = login
        self.server = server
        self.password = password
        self._message_cache = {}

    def getLatestTops(self,maxcount=20):
        """ Returns the latest tops """

        result = ()
        try:
            counter = self.getPop3().stat()[0]
            result = ( counter ,)
            while counter > 0 and maxcount >= 0:
                result += ( self._top(counter) ,)
                maxcount -= 1
                counter -= 1
        finally:
            self.disconnect()
        return result

    def getMessage(self, id):
        """ Returns the message using its server id """
        id = int(id)
        if not self._message_cache.has_key(id):
            result = ()
            try:
                result = self._retr(id)
            finally:
                self.disconnect()
            self._message_cache[id] = result
        return self._message_cache[id]

    def sendMessage(self,by,to,subject,content):
        """ Sends a message """

        smtp = smtplib.SMTP(self.server)
        try:
            smtp.sendmail(by, to, "Subject: %s\nTo: %s\n\n%s" % (subject, to, content))
        finally:
            smtp.quit()

    def deleteMessage(self,id):
        id = int(id)
        try:
            cache = {}
            for cache_id in self._message_cache.keys():
                new_id = cache_id
                if int(cache_id) > int(id):
                    new_id = int(cache_id)-1
                if int(cache_id) != int(id):
                    mail = self._message_cache[cache_id]
                    mail.id = new_id
                    cache[mail.id] = mail
            self._message_cache = cache
            id = int(id)
            counter = self.getPop3().dele(id)
        finally:
            self.disconnect()


    # --- Merely private ---

    def getPop3(self):
        if not hasattr(self,'_v_pop3'):
            self.connect()
        return self._v_pop3
        
    def connect(self):
        self._v_pop3 = poplib.POP3(self.server)
        self._v_pop3.user(self.login)
        self._v_pop3.pass_(self.password)

    def disconnect(self):
        if hasattr(self,'_v_pop3'):
            self._v_pop3.quit()
            del self._v_pop3

    def _retr(self,id):
        message = ''
        for line in self.getPop3().retr(id)[1]:
            message += line
            message += '\n'
        return Mail(message,id)

    def _top(self,id,howmuch=5):
        message = ''
        for line in self.getPop3().top(id,howmuch)[1]:
            message += line
            message += '\n'
        return Mail(message,id)


# --------------------------------
# the demo !
# 
# See an extract of all your mails
# and attachments
# --------------------------------
if __name__ == '__main__':
    import sys
    if len(sys.argv) <= 2:
        print "Usage: %s login password" % sys.argv[0]
        sys.exit(1)
    m = MailAccount('127.0.0.1',sys.argv[1],sys.argv[2])
    tops = m.getLatestTops()
    for line in map(str,tops):
        print '>>',line,'\n'
    print m.getPop3().quit()


        

